﻿using Syncfusion.Windows.Shared;
using Syncfusion.Windows.Tools.Controls;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace TreeViewAdvSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = new ViewModel();
        }
    }

    public class ViewModel
    {
        public ViewModel()
        {
          ListProjects = new ObservableCollection<Model>();
          for(int i=1; i<=5; i++ )
          {
             ListProjects.Add(new Model() { Header = "Root" + i });
          }
          MouseUpCommand = new DelegateCommand<object>(ExecuteMouseUpCommandChanged);
        }
        public DelegateCommand<object> MouseUpCommand { get; set; }
       
        public void ExecuteMouseUpCommandChanged(object parameter)
        {
            MessageBox.Show("MouseClicked on "+((parameter as TreeViewItemAdv).Header as Model).Header);

        }
        public ObservableCollection<Model> ListProjects { get; set; }
    }

    public class Model
    {
        public Model()
        {
            SubItems = new ObservableCollection<Model>();
        }
        public string Header { get; set; }
        public ObservableCollection<Model> SubItems { get; set; }
    }

    //Event to Command Behavior
    public class EventToCommand : Behavior<FrameworkElement>
    {
        private Delegate _handler;
        private EventInfo _oldEvent;

        public string EventName
        {
            get { return (string)GetValue(EventProperty); }
            set
            {
                SetValue(EventProperty, value);
            }
        }
        public static readonly DependencyProperty EventProperty = DependencyProperty.Register("EventName", typeof(string), typeof(EventToCommand), new PropertyMetadata(null, OnEventChanged));

        public ICommand Command
        {
            get { return (ICommand)GetValue(CommandProperty); }
            set
            {
                SetValue(CommandProperty, value);
            }
        }
        public static readonly DependencyProperty CommandProperty = DependencyProperty.Register("Command", typeof(ICommand), typeof(EventToCommand), new PropertyMetadata(null));



        private static void OnEventChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var eventdata = (EventToCommand)d;

            if (eventdata.AssociatedObject != null)
                eventdata.AttachHandler((string)e.NewValue);
        }

        protected override void OnAttached()
        {
            AttachHandler(this.EventName);
        }


        private void AttachHandler(string eventName)
        {

            if (_oldEvent != null)
                _oldEvent.RemoveEventHandler(this.AssociatedObject, _handler);


            if (!string.IsNullOrEmpty(eventName))
            {
                TreeViewItemAdv item = VisualUtils.FindAncestor(this.AssociatedObject, typeof(TreeViewItemAdv)) as TreeViewItemAdv;
                EventInfo ei = null;
                if (item != null)
                {
                    ei = item.GetType().GetEvent(eventName);
                }
                if (ei != null)
                {
                    MethodInfo mi = this.GetType().GetMethod("ExecuteCommand", BindingFlags.Instance | BindingFlags.NonPublic);
                    _handler = Delegate.CreateDelegate(ei.EventHandlerType, this, mi);
                    ei.AddEventHandler(item, _handler);
                    _oldEvent = ei;
                }

            }
        }


        private void ExecuteCommand(object sender, EventArgs e)
        {
            object parameter = null;
            if (parameter == null)
            {
                parameter = sender;
                
            }
            if (this.Command != null)
            {
                if (this.Command.CanExecute(parameter))
                    this.Command.Execute(parameter);
            }
        }
    }
}
